#include <syslog.h>
#include "pppd.h"
#include "upap.h"
#include "tac_lib.h"
#include "tac_plus.h"
#include "pppdtac_plus.h"


int tacacs_plus_logged_in = FALSE;
int tac_bytes_in = 0;
int tac_bytes_out = 0;
int tac_paks_in = 0;
int tac_paks_out = 0;
static time_t stime;
static char tac_user[256];
char tac_addr[256];


int tacacs_plus_login(char *user, char *passwd) {
    char tid[256];
    char author[256];
    char *tty;
    struct utmp utmp;

    strncpy(tac_user, user, sizeof(tac_user));
    tty = devnam;
    if(strncmp(tty, "/dev/", 5) == 0)
	tty += 5;

    if(!tacacs_plus_auth_pap(tac_user, passwd, tty)) {
	syslog(LOG_WARNING, "upap TACACS+ login failure for %s", tac_user);
	return UPAP_AUTHNAK;
    };

    sprintf(author, "/etc/ppp/author %s %s", tac_user, tty);
    if(system(author) != 0) {
	syslog(LOG_WARNING, "authorization failure for %s", tac_user);
	return UPAP_AUTHNAK;
    };

    stime = time(NULL);

    sprintf(tid, "task_id=%d", getpid());
    tacacs_plus_acct(tac_user, tty, TAC_PLUS_ACCT_FLAG_START, tid, "service=ppp", NULL);

    memset((void *)&utmp, 0, sizeof(utmp));
    (void)time(&utmp.ut_time);
    (void)strncpy(utmp.ut_name, tac_user, sizeof(utmp.ut_name));
    (void)strncpy(utmp.ut_host, ":PPP", sizeof(utmp.ut_host));
    (void)strncpy(utmp.ut_line, tty, sizeof(utmp.ut_line));
    login(&utmp);

    tacacs_plus_logged_in = TRUE;
    return UPAP_AUTHACK;
};


void tacacs_plus_logout() {
    char tid[256];
    char etime[256];
    char bytes_in[256], bytes_out[256];
    char paks_in[256], paks_out[256];
    char *tty;

    tty = devnam;
    if(strncmp(tty, "/dev/", 5) == 0)
	tty += 5;

    sprintf(tid, "task_id=%d", getpid());
    sprintf(etime, "elapsed_time=%d", time(NULL) - stime);
    sprintf(bytes_in, "bytes_in=%d", tac_bytes_in);
    sprintf(bytes_out, "bytes_out=%d", tac_bytes_out);
    sprintf(paks_in, "paks_in=%d", tac_paks_in);
    sprintf(paks_out, "paks_out=%d", tac_paks_out);

    tacacs_plus_acct(tac_user, tty, TAC_PLUS_ACCT_FLAG_STOP, tid, "service=ppp", "protocol=ip", tac_addr, bytes_in, bytes_out, paks_in, paks_out, etime, NULL);

    logwtmp(tty, "", "");
    logout(tty);

    tacacs_plus_logged_in = FALSE;
};
